# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist
from driverng.drivers.starexp_astm_spec import (
    StarexpSegmentO,
    StarexpSegmentR,
    StarexpSegmentL,
    Driver as DriverByg,
)
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    HprimString,
    HprimDate,
    HprimForeign,
    HprimList,
    HprimMultiple,
)


class SpaPlusSegmentH(HeaderSegment):
    sender_code = HprimString((5, 1), default='HOST')
    sender_name = HprimString((5, 2), default='P_1')
    receiver_code = HprimString((10, 1))
    receiver_name = HprimString((10, 2))
    processing_id = HprimString(12, default='P')
    version = HprimString(13, default="1")
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")
    end = HprimForeign(
        "L", "header", one2one=True, required=True
    )


class SpaPlusSegmentO(StarexpSegmentO):
    specimen_id = HprimString((3, 1))
    rack = HprimString((4, 1))
    position = HprimString((4, 2))
    tests = HprimList(5, basefield=HprimMultiple(
        0,
        code=HprimString(4),
        name=HprimString(5),
        dilution=HprimString(7),
    ))
    priority = HprimString(6, default='R')
    collection_date = HprimDate(8)
    action_code = HprimString(12, default='N')
    specimen = HprimString(16)
    physician_id = HprimString((17, 1))
    physician_name = HprimString((17, 2))
    service_id = HprimString((17, 4))
    service_name = HprimString((17, 5))
    report_type = HprimString(26)
    operator = HprimString(33)

    results = HprimForeign("R", "order")


class SpaPlusSegmentR(StarexpSegmentR):
    test = HprimString((3, 4))
    test_name = HprimString((3, 5))
    dilution = HprimString((3, 7))
    value = HprimString(4)
    units = HprimString(5)
    flag = HprimString(7)
    # F: Final
    status = HprimString(9)
    operator = HprimString(11)
    date = HprimDate(13)
    instrument = HprimString(14)
    reagent_kit = HprimString(17)


class SpaPlusSegmentL(StarexpSegmentL):
    pass


class Driver(DriverByg):
    SPEC = {
        "code": "spa_plus",
        "label": "Spa Plus Binding Site",
    }
    SEGMENT_CLASS_MAP = DriverByg.SEGMENT_CLASS_MAP.copy()
    SEGMENT_CLASS_MAP.update(**{
        "H": SpaPlusSegmentH,
        "O": SpaPlusSegmentO,
        "R": SpaPlusSegmentR,
    })
    OPTIONS = [
        AstmMessageV2.Option(
            'spa_plus_sender_code',
            label="Indiquer le code de l'émetteur (champs 5.1)"),
        AstmMessageV2.Option(
            'spa_plus_sender_name',
            label="Indiquer le nom de l'émetteur (champs 5.2)"),
        AstmMessageV2.Option(
            'spa_plus_receiver_code',
            label='Indiquer le code du récepteur (champs 10.1)'),
        AstmMessageV2.Option(
            'spa_plus_receiver_name',
            label='Indiquer le nom du récepteur (champs 10.2)'),
        AstmMessageV2.Option(
            "spa_plus_qc_prefix", label="Préfixe QC"),
    ]
    DEFAULT_SEPARATORS = "|\\^&"

    def __init__(
            self, spa_plus_sender_code="", spa_plus_sender_name="",
            spa_plus_receiver_code="", spa_plus_receiver_name="",
            spa_plus_qc_prefix="",
            **kwargs):
        self.spa_plus_sender_code = spa_plus_sender_code
        self.spa_plus_sender_name = spa_plus_sender_name
        self.spa_plus_receiver_code = spa_plus_receiver_code
        self.spa_plus_receiver_name = spa_plus_receiver_name
        self.qc_prefix = spa_plus_qc_prefix
        super(DriverByg, self).__init__(**kwargs)

    def create(self, **kwargs):
        message = SpaPlusSegmentH(type_segment="H", **kwargs)
        message.end = SpaPlusSegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        msg = self.create(
            date=context["now"],
            sender_code=self.spa_plus_sender_code,
            sender_name=self.spa_plus_sender_name,
            receiver_code=self.spa_plus_receiver_code,
            receiver_name=self.spa_plus_receiver_name
        )
        msg.sender_code = self.spa_plus_sender_code
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            action={"del": "C", worklist.ELSE: "N"},
            priority={"S": "S", worklist.ELSE: "R"},
        ):
            if patient.services:
                service_code = patient.services[0].code
            else:
                service_code = ''

            seg_p = msg.patients.add(
                patient_id=patient.patient_id,
                last_name=patient.last_name,
                first_name=patient.first_name,
                nomjf=patient.maiden_name,
                birthdate=patient.birthdate,
                sex=patient.sex,
                height=patient.height,
                weight=patient.weight,
                service=service_code,
                comment1="{:.60s}".format(patient.patient_comment1),
            )
            if patient.medications:
                medication = seg_p.medication.add()
                if len(patient.medications) > 1:
                    medication.code = ";".join(patient.medications)
                else:
                    medication.code = patient.medications
            for sample, action, priority in patient.samples:
                seg_o = seg_p.orders.add(
                    specimen_id=sample.sample_id,
                    priority=priority,
                    action_code=action,
                    collection_date=sample.request_date,
                    specimen=sample.specimen,
                    service_id=service_code,
                )
                for physician in sample.physicians:
                    seg_p.physician_id = physician.code
                    seg_p.physician_name = physician.last_name
                    seg_o.physician_id = physician.code
                    seg_o.physician_name = physician.last_name
                for order in sample.orders:
                    test = seg_o.tests.add()
                    test.code = order.code
                    test.dilution = order.dilution

        if msg.patients:
            return msg

    def decode_message(self, message):
        worklist = Worklist(context="result")
        for seg_p in message.patients:
            for seg_o in seg_p.orders:
                control = message.processing_id == "Q" or (self.qc_prefix and seg_o.specimen_id.startswith(self.qc_prefix))
                if control:
                    sample = worklist.get_or_create(
                        sample_id=seg_o.specimen_id,
                        type="control",
                        control_lot=seg_o.specimen_id,
                    )
                else:
                    sample = worklist.get_or_create(
                        sample_id=seg_o.specimen_id,
                        type="patient",
                    )

                if seg_o.rack and seg_o.position:
                    sample.rack = "%s-%s" % (seg_o.rack, seg_o.position)
                if seg_o.specimen:
                    sample.specimen = seg_o.specimen
                for seg_r in seg_o.results:
                    # uniquement les resultats avec le statut F
                    if seg_r.status != "F":
                        continue
                    commentaires = []
                    for seg_c in seg_r.commentaires:
                        commentaires.extend([c for c in [
                            seg_c.commentaire,
                            seg_c.comment_long1,
                            seg_c.comment_long2,
                            seg_c.comment_long3,
                            seg_c.comment_long4,
                            seg_c.comment_long5]
                            if c])
                    result = sample.results.add(
                        code=seg_r.test,
                        value=seg_r.value,
                        date=seg_r.date,
                        unit=seg_r.units,
                        flags=[seg_r.flag],
                        dilution=seg_r.dilution,
                        instrument=seg_r.instrument,
                        comments=commentaires,
                    )
                    if seg_r.reagent_kit:
                        result.lots.add(type="reagent", lot=seg_r.reagent_kit)
                    if sample.type == 'patient' and seg_r.operator:
                        result.operator = seg_r.operator
        return worklist
