# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimString,
    HprimList,
    HprimMultiple,
    HprimDate,
    HprimForeign,
)
from driverng.utils import num_years


class YhloSegmentH(HeaderSegment):
    sender_id = HprimString((5, 1))
    receiver_id = HprimString((10, 1))
    processing_id = HprimString(12)
    version = HprimString(13, default="1394-97")
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class YhloSegmentQ(Segment):
    patient_id = HprimString((3, 1))
    specimen_id = HprimString((3, 2))
    ending_range_id = HprimString(4)
    universal_test_id = HprimString(5)
    beginning_request_results_date = HprimString(7)
    ending_request_results_date = HprimString(8)
    request_information = HprimString(13)


class YhloSegmentP(Segment):
    patient_id = HprimString(4)
    laboratory_patient_id = HprimString(5)
    last_name = HprimString((6, 1))
    first_name = HprimString((6, 2))
    birthdate = HprimDate((8, 1), fmt_date="%Y%m%d")
    age = HprimString((8, 2))
    age_unit = HprimString((8, 3))
    sex = HprimString(9)
    attending_physician = HprimString(14)
    body_surface = HprimString(16)
    patient_diet = HprimString(21)
    diagnosis = HprimString(27)

    orders = HprimForeign("O", "patient")


class YhloSegmentO(Segment):
    sample_id = HprimString((3, 1))
    rack = HprimString((3, 2))
    position = HprimString((3, 3))
    instrument_specimen_id = HprimString(4)
    test = HprimList(5, basefield=HprimMultiple(
        0,
        assay_no=HprimString(1),
        assay_name=HprimString(2),
        dilution_rate=HprimString(3),
        repeat_num=HprimString(4),
        rerun_flag=HprimString(5)
    ))
    # R: Normal
    # S: STAT
    priority = HprimString(6)
    requested_date = HprimDate(7)
    collection_date = HprimDate(8)
    collection_end_time = HprimDate(9)
    collection_volume = HprimString(10)
    collected_by = HprimString(11)
    control = HprimList(12, basefield=HprimMultiple(
        0,
        control_number=HprimString(1),
        control_name=HprimString(2),
        period_validity=HprimDate(3),
        average_concentration=HprimString(4),
        level=HprimString(5),
        standard_deviation=HprimString(6),
        concentration=HprimString(7)
    ))
    danger_code = HprimString(13)
    relevant_clinical_information = HprimString(14)
    specimen_type = HprimString(16)
    ordering_physician = HprimString(17)
    report_type = HprimString(26)

    results = HprimForeign("R", "order")


class YhloSegmentR(Segment):
    assay_no = HprimString((3, 1))
    assay_name = HprimString((3, 2))
    replicate_number = HprimString((3, 3))
    result_type = HprimString((3, 4))
    value = HprimString((4, 1))
    interpretation = HprimString((4, 2))
    units = HprimString(5)
    measurement_range_upper_limit = HprimString((6, 1))
    measurement_range_lower_limit = HprimString((6, 2))
    abnormal_flag = HprimString(7)
    nature_abnormality_testing = HprimString(8)
    result_status = HprimString(9)
    operator = HprimString(11)
    date_test_started = HprimDate(12)
    date_test_completed = HprimDate(13)
    instrument = HprimString(14)


class YhloSegmentC(Segment):
    comment_source = HprimString(3)
    comment_text = HprimString(4)
    comment_type = HprimString(5)


class YhloSegmentL(Segment):
    terminator_code = HprimString(3)


class Driver(AstmMessageV2):
    SPEC = {
        "code": "yhlo",
        "label": "Yhlo ASTM ORGENTEC",
    }
    SEGMENT_CLASS_MAP = {
        "H": YhloSegmentH,
        "Q": YhloSegmentQ,
        "P": YhloSegmentP,
        "O": YhloSegmentO,
        "R": YhloSegmentR,
        "C": YhloSegmentC,
        "L": YhloSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"
    OPTIONS = [
        AstmMessageV2.Option(
            "yhlo_sender_id",
            label="Code émetteur H.5",
            default=""
        ),
        AstmMessageV2.Option(
            "yhlo_receiver_id",
            label="Code récepteur H.10",
            default=""
        ),
    ]

    def __init__(self, **kwargs):
        self.sender_id = kwargs.pop("yhlo_sender_id", "")
        self.receiver_id = kwargs.pop("yhlo_receiver_id", "")
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        message = YhloSegmentH(type_segment="H", **kwargs)
        message.end = YhloSegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        message = self.create(
            sender_id=self.sender_id,
            receiver_id=self.receiver_id,
            date=context["now"],
            processing_id="SA"
        )
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            priority={"A": "S", "S": "S", worklist.ELSE: "R"},
            action={"del": worklist.IGNORE, worklist.ELSE: ""},
        ):
            age = num_years(patient.birthdate, patient.collection_date)
            if not age:
                age = ""
                age_unit = ""
            else:
                age_unit = "Y"
            seg_p = message.patients.add(
                patient_id=patient.patient_id,
                laboratory_patient_id=patient.lab_patient_id,
                last_name=patient.last_name,
                first_name=patient.first_name,
                birthdate=patient.birthdate,
                age=age,
                age_unit=age_unit,
                sex=patient.sex,
            )
            for sample, action, priority in patient.samples:
                if sample.diagnostic:
                    seg_p.diagnosis = sample.diagnostic
                seg_o = seg_p.orders.add(
                    instrument_specimen_id=sample.sample_id,
                    priority=priority,
                    requested_date=sample.request_date,
                    collection_date=sample.collection_date,
                    specimen_type=sample.specimen,
                    report_type="Q" if context["query"] else "O",
                )
                for physician in sample.physicians:
                    if physician.type == "prescriber":
                        seg_o.ordering_physician = physician.code
                        break
                    elif physician.type == "attending":
                        seg_p.attending_physician = physician.code
                for order in sample.orders:
                    t = seg_o.test.add()
                    t.assay_name = order.code
        return message

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                worklist.get_or_create(
                    type="patient",
                    sample_id=seg_q.specimen_id,
                )
        else:
            worklist = Worklist(context="result")
            for seg_p in message.patients:
                for seg_o in seg_p.orders:
                    if message.processing_id == "QR":
                        # QC results
                        if seg_o.control:
                            for control in seg_o.control:
                                sample = worklist.get_or_create(
                                    sample_id=control.control_number,
                                    control_lot=control.control_number,
                                    control_name=control.control_name,
                                    specimen=seg_o.specimen_type,
                                    type="control"
                                )
                                sample.results.add(
                                    code="TEST",
                                    value=control.concentration,
                                    date=message.date
                                )
                    else:
                        sample = worklist.get_or_create(
                            sample_id=seg_o.instrument_specimen_id,
                            specimen=seg_o.specimen_type,
                            rack="{}{}".format(seg_o.rack, seg_o.position),
                            type="patient"
                        )
                    for seg_r in seg_o.results:
                        comments = []
                        flags = []
                        for seg_c in seg_r.commentaires:
                            comments.append(seg_c.comment_text)
                        if seg_r.abnormal_flag:
                            flags.append(seg_r.abnormal_flag)
                        # Résultat numérique
                        if seg_r.value:
                            sample.results.add(
                                code=seg_r.assay_no,
                                value=seg_r.value,
                                flags=flags,
                                comments=comments,
                                unit=seg_r.units,
                                date=seg_r.date_test_completed,
                                operator=seg_r.operator,
                                instrument=seg_r.instrument,
                            )
                        # Interprétation
                        if seg_r.interpretation:
                            sample.results.add(
                                code=seg_r.assay_name,
                                value=seg_r.interpretation,
                                flags=flags,
                                comments=comments,
                                unit=seg_r.units,
                                date=seg_r.date_test_completed,
                                operator=seg_r.operator,
                                instrument=seg_r.instrument,
                            )
        return worklist
