# -*- coding: utf8 -*-
from __future__ import unicode_literals

import xml.etree.cElementTree as ET
import datetime

from driverng import MessageV2
from driverng import utils
from driverng import Worklist


class Driver(MessageV2):
    SPEC = {
        "code": "zenitupxml",
        "label": "ZENITUP XML DAS"
    }
    ENCODING = "utf8"

    def parse(self, data):
        dataxml = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\r\n" + data
        return ET.fromstring(dataxml.encode(self.ENCODING))

    def to_string(self, message):
        message_xml = utils.prettify_xml(ET.tostring(message, 'utf-8'))
        return "\r\n".join(message_xml.splitlines()[1:])

    def encode_worklist(self, worklist, context):
        message = ""
        message = ET.Element("GROUP")
        message.set("NAME", "WORKLIST")
        message.set("TYPE", "HOST")
        for sample, action, priority in worklist.get_worklist(
            unique_together=["action", "priority"],
            action={"del": worklist.IGNORE, worklist.ELSE: ""},
            priority={"A": "S", "S": "S", worklist.ELSE: "R"},
        ):
            group_sample = ET.SubElement(message, "GROUP")
            group_sample.set("TYPE", "Sample")
            group_patient = ET.SubElement(group_sample, "GROUP")
            # Tube
            group_patient.set("TYPE", "Patient")
            group_patient.set("ID", sample.sample_id)

            # Nom et prénom du patient sur 32 caractères maxi
            param_patient_name = ET.SubElement(group_patient, "PARAM")
            patient_name = "{} {}".format(sample.last_name, sample.first_name)
            param_patient_name.set("TYPE", "{:.32s}".format(patient_name))
            param_patient_name.set("ID", "Name")

            # Date de naissance
            param_patient_birthdate = ET.SubElement(group_patient, "PARAM")
            param_patient_birthdate.set("TYPE", utils.format_date(sample.birthdate, fmt="%Y%m%d"))
            param_patient_birthdate.set("ID", "BirthDate")

            # Sexe
            param_patient_sex = ET.SubElement(group_patient, "PARAM")
            param_patient_sex.set("TYPE", sample.sex)
            param_patient_sex.set("ID", "Sex")

            # Tests
            if sample["orders"]:
                group_orders = ET.SubElement(group_sample, "GROUP")
                group_orders.set("TYPE", "Assays")
                group_orders.set("ID", "")
                for order in sample["orders"]:
                    # Test
                    group_order = ET.SubElement(group_orders, "GROUP")
                    group_order.set("TYPE", "Assay")
                    group_order.set("ID", order.code)
                    group_order.set("STATE", "Host")
                    # Dilution
                    if order.dilution:
                        group_dilutions = ET.SubElement(group_order, "GROUP")
                        group_dilutions.set("TYPE", "Dilutions")
                        param_dilution = ET.SubElement(group_dilutions, "PARAM")
                        param_dilution.set("TYPE", "Dilution")
                        param_dilution.set("ID", order.dilution)

        return message

    def decode_message(self, message):
        worklist = Worklist(context="result")
        date_result = None
        instrument = ""
        for group in message.findall("GROUP"):
            if group.attrib.get("TYPE") == "Instrument":
                instrument = group.attrib.get("ID")
            if group.attrib.get("ID") == "SessionInfo":
                for param in group.findall("PARAM"):
                    if param.attrib.get("ID") == "Execution":
                        date_result = datetime.datetime.strptime(param.text, "%d/%m/%Y %H:%M:%S")
            for sub_group in group.findall("GROUP"):
                for group_assay in sub_group.findall("GROUP"):
                    test_code = ""
                    if group_assay.attrib.get("TYPE") == "Assay":
                        test_code = group_assay.attrib.get("ID")
                        for group_patient in group_assay.findall("GROUP"):
                            sample = worklist.get_or_create(
                                type="patient",
                                sample_id=group_patient.attrib.get("ID")
                            )
                            for position in group_patient.findall("GROUP"):
                                if position.attrib.get("TYPE") == "Position":
                                    rack = "Position {}".format(position.attrib.get("ID"))
                                    sample.rack = rack
                            for param in group_patient.findall("PARAM"):
                                # Value
                                if param.attrib.get("ID") == "Value":
                                    sample.results.add(
                                        code=test_code,
                                        value=param.text,
                                        instrument=instrument,
                                        date=date_result
                                    )
                                # Résultat de concentration
                                if param.attrib.get("ID") == "Concentration":
                                    sample.results.add(
                                        code="{}_CONC".format(test_code),
                                        value=param.text,
                                        instrument=instrument,
                                        date=date_result
                                    )
                                # Résultat interprété
                                if param.attrib.get("ID") == "Result":
                                    sample.results.add(
                                        code="{}_I".format(test_code),
                                        value=param.text,
                                        instrument=instrument,
                                        date=date_result
                                    )
        return worklist
